# -*- coding: utf-8 -*-
"""
Created on Fri Apr  5 12:47:15 2024

@author: kristian
"""

import numpy as np
import matplotlib.pyplot as plt
from netCDF4 import Dataset
from wrf import (getvar, to_np, get_cartopy, latlon_coords, vertcross,ALL_TIMES,cartopy_xlim, cartopy_ylim, interpline, CoordPair,xy_to_ll,ll_to_xy,interplevel)
from numpy.fft import fft

wrf_file=[Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_00_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_01_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_02_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_03_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_04_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_05_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_06_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_07_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_08_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_09_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_10_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_11_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_12_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_13_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_14_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_15_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_16_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_17_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_18_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_19_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_20_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_21_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_22_00_00.nc"),
            Dataset(r"H:\CSCS\WRF_dia1103\wrfout_d01_2021-03-11_23_00_00.nc")]

#coordinate start and end
cross_start = CoordPair(lat=46.29318, lon=7.03207) #west:46.29318, 7.03207
cross_end = CoordPair(lat=46.38629, lon=7.39801) #east:46.38629, 7.39801

id_time=0

ht = getvar(wrf_file, "z", timeidx=ALL_TIMES)
ter = getvar(wrf_file, "ter", timeidx=ALL_TIMES)
ua = getvar(wrf_file, "ua", timeidx=ALL_TIMES)
va = getvar(wrf_file, "va", timeidx=ALL_TIMES)
wa = getvar(wrf_file, "wa", timeidx=ALL_TIMES)
PBLH = getvar(wrf_file, "PBLH", timeidx=ALL_TIMES)
u10=getvar(wrf_file, "U10")
v10=getvar(wrf_file, "V10", timeidx=id_time)
pottemp = getvar(wrf_file, "th", timeidx=ALL_TIMES,units="K")
 

end_index=30
ht_cut=ht[0:end_index,:,:]
ua_cut = ua[0:end_index,:,:]
wa_cut = wa[0:end_index,:,:]
uvw=(ua**2+va**2+wa**2)**0.5
uvw_cut=uvw[1:end_index,:,:]
uv=(ua**2+va**2)**0.5
uv_cut=uv[0:end_index,:,:]
th_cut=pottemp[0:end_index,:,:]

uv3000 = interplevel(uv,ht, 3000) #uv at ht 3000m

# ######################vertical wind speed#####################################################
#cut wa_cut by start_point to end_point. z_cross[24,100,100] [time,cross section, height]
z_cross = vertcross(wa_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)
w_cross = vertcross(wa, ht, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)

ua_np = np.ma.copy(to_np(z_cross))

ter_line = interpline(ter, wrfin=wrf_file, start_point=cross_start,end_point=cross_end)
xs = np.arange(0, z_cross.shape[-1], 1)
ys = to_np(z_cross.coords["vertical"]) #height level of z_cross (100)


plt.figure(figsize=(12,4))
dbz_levels = np.arange(-10,10,0.1)
plt.contourf(z_cross[:,19,:],levels=dbz_levels,extend="max",cmap="bwr") #cross section at 2564.9023 magl
cbar=plt.colorbar()
cbar.ax.tick_params(labelsize=15)
cbar.ax.set_ylabel('Vertical wind speed (m/s)', fontsize=15)
coord_pairs = to_np(z_cross.coords["xy_loc"])
x_ticks = np.arange(coord_pairs.shape[0])
x_labels = [pair.latlon_str() for pair in to_np(coord_pairs)]
num_ticks = 10
thin = int((len(x_ticks) / num_ticks) + .5)
plt.xticks(x_ticks[::thin],labels=x_labels[::thin], rotation=45, fontsize=15)
plt.plot([50,50],[0,23],'k-')
plt.plot([0,99],[10,16],'b-.')
plt.xlabel("Latitude, Longitude", fontsize=15)
plt.ylabel("Hours", fontsize=15)
plt.tick_params(labelsize=15)
# plt.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_dia_11_03\figure\hovmoller\vertical_windspeed_titled.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
plt.show()

plt.figure(figsize=(12,4))
dbz_levels = np.arange(-10,10,0.1)
plt.contourf(np.arange(0,24),ys,np.transpose(np.array(z_cross[:,:,50])),levels=dbz_levels,extend="max",cmap="bwr")
cbar=plt.colorbar()
cbar.ax.tick_params(labelsize=15)
cbar.ax.set_ylabel('Vertical wind speed (m/s)', fontsize=15)
plt.ylim(2500,5000)
# Set the x-axis and  y-axis labels
plt.xlabel("Hours", fontsize=15)
plt.ylabel("Height (m.a.s.l)", fontsize=15)
plt.tick_params(labelsize=15)
# plt.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_dia_11_03\figure\hovmoller\vertical_windspeed_lidar_location_titled.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
plt.show()

#######################horizontal wind speed#####################################################
z_cross = vertcross(uv, ht, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)
w_cross = vertcross(wa, ht, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)

ua_np = np.ma.copy(to_np(z_cross))

ter_line = interpline(ter, wrfin=wrf_file, start_point=cross_start,end_point=cross_end)
xs = np.arange(0, z_cross.shape[-1], 1)
ys = to_np(z_cross.coords["vertical"])


plt.figure(figsize=(12,4))
dbz_levels = np.arange(0,33,0.1)
plt.contourf(z_cross[:,19,:],levels=dbz_levels,extend="max",cmap="rainbow")
cbar=plt.colorbar()
cbar.ax.tick_params(labelsize=15)
cbar.ax.set_ylabel('Horizontal wind speed (m/s)', fontsize=15)
coord_pairs = to_np(z_cross.coords["xy_loc"])
x_ticks = np.arange(coord_pairs.shape[0])
x_labels = [pair.latlon_str() for pair in to_np(coord_pairs)]
num_ticks = 10
thin = int((len(x_ticks) / num_ticks) + .5)
plt.xticks(x_ticks[::thin],labels=x_labels[::thin], rotation=45, fontsize=15)
plt.plot([50,50],[0,23],'k-')
plt.plot([0,99],[10,16],'b-.')
plt.xlabel("Latitude, Longitude", fontsize=15)
plt.ylabel("Hours", fontsize=15)
plt.tick_params(labelsize=15)
# plt.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_dia_11_03\figure\hovmoller\uv_windspeed_titled.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
plt.show()

plt.figure(figsize=(12,4))
dbz_levels = np.arange(0,40,0.1)
plt.contourf(np.arange(0,24),ys,np.transpose(np.array(z_cross[:,:,50])),levels=dbz_levels,extend="max",cmap="rainbow")
cbar=plt.colorbar()
cbar.ax.tick_params(labelsize=15)
cbar.ax.set_ylabel('Horizontal wind speed (m/s)', fontsize=15)
plt.ylim(2500,5000)
# Set the x-axis and  y-axis labels
plt.xlabel("Hours", fontsize=15)
plt.ylabel("Height (m.a.s.l.)", fontsize=15)
plt.tick_params(labelsize=15)
#plt.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_dia_11_03\figure\hovmoller\uv_windspeed_lidar_location_titled.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
plt.show()
#######################potential temperature and horizontal wind speed#####################################################
#lidar vertical cross section wave vertical velocity 08h00
hour=8
z_cross = vertcross(uv_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)[hour,:,:]
th_cross = vertcross(th_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)[hour,:,:]

ua_np = np.ma.copy(to_np(z_cross))

ter_line = interpline(ter, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)

PBLH_line = interpline(PBLH, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)
PBLH_line_ter=PBLH_line+ter_line

lats, lons = latlon_coords(ua)
cart_proj = get_cartopy(ua)

fig = plt.figure(figsize=(12,4))
ax_cross = plt.axes()
dbz_levels = np.arange(0,25,0.1)
xs = np.arange(0, z_cross.shape[-1], 1)
ys = to_np(z_cross.coords["vertical"])
dbz_contours = ax_cross.contourf(xs,ys,to_np(z_cross),levels=dbz_levels,extend="max",cmap="rainbow")
CS =ax_cross.contour(xs,ys,th_cross,levels=[i for i in np.arange(290,320,2)])
ax_cross.clabel(CS, inline=True, fontsize=15)

interval_x=10
interval_y=5

#add colorbar
cb_dbz = fig.colorbar(dbz_contours, ax=ax_cross)
cb_dbz.ax.tick_params(labelsize=15)
cb_dbz.ax.set_ylabel('Horizontal wind speed (m/s)', fontsize=15)

# Fill in the mountain area
ht_fill = ax_cross.fill_between(xs, 0, to_np(ter_line)[hour,:],facecolor="saddlebrown")
coord_pairs = to_np(z_cross.coords["xy_loc"])
x_ticks = np.arange(coord_pairs.shape[0])
x_labels = [pair.latlon_str() for pair in to_np(coord_pairs)]

# Set the desired number of x ticks below
num_ticks = 10
thin = int((len(x_ticks) / num_ticks) + .5)
ax_cross.set_xticks(x_ticks[::thin])
ax_cross.set_xticklabels(x_labels[::thin], rotation=45, fontsize=10)

# Set the x-axis and  y-axis labels
ax_cross.set_xlabel("Latitude, Longitude", fontsize=15)
ax_cross.set_ylabel("Height (m.a.s.l.)", fontsize=15)
ax_cross.tick_params(labelsize=15)
plt.plot(50,2350,'ko',markersize=10)
plt.ylim(0,5000)
# pyplot.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_dia_11_03\figure\xz\xz_uv_11_08h_pottemp_clabel1.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
plt.show()