# -*- coding: utf-8 -*-
"""
Created on Fri Apr  5 14:04:04 2024

@author: kristian
"""
import numpy as np
from matplotlib import pyplot
from netCDF4 import Dataset
from wrf import (getvar, to_np, get_cartopy, latlon_coords, vertcross,ALL_TIMES,cartopy_xlim, cartopy_ylim, interpline, CoordPair,xy_to_ll,ll_to_xy)

# %% 23/10/2020 SOUTHERLY FOEHN
wrf_file=Dataset(r"H:\CSCS\WRF_luk2310\wrfout_d01_2020-10-23_13_00_00.nc")

#GOS-GUTSCH
cross_start = CoordPair(lat=46.64560, lon=8.61934) #gutsch:46.64560, 8.61934
cross_end = CoordPair(lat=46.71181, lon=8.58495) #gos:46.71181, 8.58495

id_time=0

ht = getvar(wrf_file, "z", timeidx=id_time)
ter = getvar(wrf_file, "ter", timeidx=id_time)
ua = getvar(wrf_file, "ua", timeidx=id_time)
va = getvar(wrf_file, "va", timeidx=id_time)
wa = getvar(wrf_file, "wa", timeidx=id_time)
PBLH = getvar(wrf_file, "PBLH", timeidx=id_time)
u10=getvar(wrf_file, "U10")
v10=getvar(wrf_file, "V10", timeidx=id_time)
pottemp = getvar(wrf_file, "th", timeidx=id_time,units="K")
tk = getvar(wrf_file, "tk", timeidx=id_time)
rh = getvar(wrf_file, "rh", timeidx=id_time)

end_index=30
ht_cut=ht[0:end_index,:,:]
ua_cut = ua[0:end_index,:,:]
wa_cut = wa[0:end_index,:,:]
uvw=(ua**2+va**2+wa**2)**0.5
uvw_cut=uvw[1:end_index,:,:]
uv=(ua**2+va**2)**0.5
uv_cut=uv[0:end_index,:,:]
th_cut=pottemp[0:end_index,:,:]
tk_cut=tk[0:end_index,:,:]
rh_cut=rh[0:end_index,:,:]

# %% HORIZONTAL WIND SPEED
z_cross = vertcross(uv_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)
th_cross = vertcross(th_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)

ua_np = np.ma.copy(to_np(z_cross))

ter_line = interpline(ter, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)

PBLH_line = interpline(PBLH, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)
PBLH_line_ter=PBLH_line+ter_line

lats, lons = latlon_coords(ua)
cart_proj = get_cartopy(ua)

fig = pyplot.figure(figsize=(12,4))
ax_cross = pyplot.axes()
dbz_levels = np.arange(8,25,0.1)  #luk 23/10/2020 southerly foehn 13h00
xs = np.arange(0, z_cross.shape[-1], 1)
ys = to_np(z_cross.coords["vertical"])
dbz_contours = ax_cross.contourf(xs,ys,to_np(z_cross),levels=dbz_levels,extend="max",cmap="rainbow")
CS =ax_cross.contour(xs,ys,z_cross,levels=[i for i in np.arange(0,30,2)])
ax_cross.clabel(CS, inline=True, fontsize=12)

interval_x=10
interval_y=5

#add colorbar
cb_dbz = fig.colorbar(dbz_contours, ax=ax_cross)
cb_dbz.ax.tick_params(labelsize=15)
cb_dbz.ax.set_ylabel('Horizontal wind speed (m/s)', fontsize=15)

# Fill in the mountain area
ht_fill = ax_cross.fill_between(xs, 0, to_np(ter_line),facecolor="saddlebrown")

# Set the x-ticks to use latitude and longitude labels
coord_pairs = to_np(z_cross.coords["xy_loc"])
x_ticks = np.arange(coord_pairs.shape[0])
x_labels = [pair.latlon_str() for pair in to_np(coord_pairs)]

# Set the desired number of x ticks below
num_ticks = 10
thin = int((len(x_ticks) / num_ticks) + .5)
ax_cross.set_xticks(x_ticks[::thin])
ax_cross.set_xticklabels(x_labels[::thin], rotation=45, fontsize=10)

# Set the x-axis and  y-axis labels
ax_cross.set_xlabel("Latitude, Longitude", fontsize=15)
ax_cross.set_ylabel("Height (m.a.s.l.)", fontsize=15)
ax_cross.tick_params(labelsize=15)
pyplot.ylim(0,5000)
# pyplot.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_luk_09_12\Figure\xz_cross_section\paper_xz_13h_uv_5000.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
pyplot.show()
# %% TEMPERATURE
z_cross = vertcross(tk_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)
th_cross = vertcross(tk_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)

ua_np = np.ma.copy(to_np(z_cross))

ter_line = interpline(ter, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)

PBLH_line = interpline(PBLH, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)
PBLH_line_ter=PBLH_line+ter_line

lats, lons = latlon_coords(ua)
cart_proj = get_cartopy(ua)

fig = pyplot.figure(figsize=(12,4))
ax_cross = pyplot.axes()
dbz_levels = np.arange(258,285,1) #luk 23/10/2020 southerly foehn 13h00
xs = np.arange(0, z_cross.shape[-1], 1)
ys = to_np(z_cross.coords["vertical"])
dbz_contours = ax_cross.contourf(xs,ys,to_np(z_cross),levels=dbz_levels,extend="max",cmap="rainbow")
CS =ax_cross.contour(xs,ys,z_cross,levels=[i for i in np.arange(240,295,2)])
ax_cross.clabel(CS, inline=True, fontsize=12)

interval_x=10
interval_y=5

#add colorbar
cb_dbz = fig.colorbar(dbz_contours, ax=ax_cross)
cb_dbz.ax.tick_params(labelsize=15)
cb_dbz.ax.set_ylabel('Temperature (K)', fontsize=15)

# Fill in the mountain area
ht_fill = ax_cross.fill_between(xs, 0, to_np(ter_line),facecolor="saddlebrown")

# Set the x-ticks to use latitude and longitude labels
coord_pairs = to_np(z_cross.coords["xy_loc"])
x_ticks = np.arange(coord_pairs.shape[0])
x_labels = [pair.latlon_str() for pair in to_np(coord_pairs)]

# Set the desired number of x ticks below
num_ticks = 10
thin = int((len(x_ticks) / num_ticks) + .5)
ax_cross.set_xticks(x_ticks[::thin])
ax_cross.set_xticklabels(x_labels[::thin], rotation=45, fontsize=10)

# Set the x-axis and  y-axis labels
ax_cross.set_xlabel("Latitude, Longitude", fontsize=15)
ax_cross.set_ylabel("Height (m.a.s.l.)", fontsize=15)
ax_cross.tick_params(labelsize=15)
pyplot.ylim(0,5000)
# pyplot.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_luk_09_12\Figure\xz_cross_section\paper_xz_13h_tk_5000.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
pyplot.show()
# %% RELATIVE HUMIDITY
z_cross = vertcross(rh_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)
th_cross = vertcross(rh_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)

ua_np = np.ma.copy(to_np(z_cross))

ter_line = interpline(ter, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)

PBLH_line = interpline(PBLH, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)
PBLH_line_ter=PBLH_line+ter_line

lats, lons = latlon_coords(ua)
cart_proj = get_cartopy(ua)

fig = pyplot.figure(figsize=(12,4))
ax_cross = pyplot.axes()
dbz_levels = np.arange(30,100,1)
xs = np.arange(0, z_cross.shape[-1], 1)
ys = to_np(z_cross.coords["vertical"])
dbz_contours = ax_cross.contourf(xs,ys,to_np(z_cross),levels=dbz_levels,extend="max",cmap="rainbow")
CS=ax_cross.contour(xs,ys,z_cross,levels=[i for i in np.arange(20,100,5)])
ax_cross.clabel(CS, inline=True, fontsize=12)

interval_x=10
interval_y=5

#add colorbar
cb_dbz = fig.colorbar(dbz_contours, ax=ax_cross)
cb_dbz.ax.tick_params(labelsize=15)
cb_dbz.ax.set_ylabel('Relative humidity (%)', fontsize=15)

# Fill in the mountain area
ht_fill = ax_cross.fill_between(xs, 0, to_np(ter_line),facecolor="saddlebrown")

# Set the x-ticks to use latitude and longitude labels
coord_pairs = to_np(z_cross.coords["xy_loc"])
x_ticks = np.arange(coord_pairs.shape[0])
x_labels = [pair.latlon_str() for pair in to_np(coord_pairs)]

# Set the desired number of x ticks below
num_ticks = 10
thin = int((len(x_ticks) / num_ticks) + .5)
ax_cross.set_xticks(x_ticks[::thin])
ax_cross.set_xticklabels(x_labels[::thin], rotation=45, fontsize=10)

# Set the x-axis and  y-axis labels
ax_cross.set_xlabel("Latitude, Longitude", fontsize=15)
ax_cross.set_ylabel("Height (m.a.s.l.)", fontsize=15)
ax_cross.tick_params(labelsize=15)
pyplot.ylim(0,5000)
# pyplot.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_luk_09_12\Figure\xz_cross_section\xz_09h_rh_5000_mid.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
pyplot.show()
# %% 09/12/2020 NORTHERLY FOEHN
wrf_file=Dataset(r"H:\CSCS\WRF_luk0912\wrfout_d01_2020-12-09_09_00_00.nc")

#GUE-BIA
cross_start = CoordPair(lat=46.66363, lon=8.60292) #GUE: 46.66363, 8.60292
cross_end = CoordPair(lat=46.43380, lon=8.86705) #BIA: 46.43380, 8.86705

id_time=0

ht = getvar(wrf_file, "z", timeidx=id_time)
ter = getvar(wrf_file, "ter", timeidx=id_time)
ua = getvar(wrf_file, "ua", timeidx=id_time)
va = getvar(wrf_file, "va", timeidx=id_time)
wa = getvar(wrf_file, "wa", timeidx=id_time)
PBLH = getvar(wrf_file, "PBLH", timeidx=id_time)
u10=getvar(wrf_file, "U10")
v10=getvar(wrf_file, "V10", timeidx=id_time)
pottemp = getvar(wrf_file, "th", timeidx=id_time,units="K")
tk = getvar(wrf_file, "tk", timeidx=id_time)
rh = getvar(wrf_file, "rh", timeidx=id_time)

end_index=30
ht_cut=ht[0:end_index,:,:]
ua_cut = ua[0:end_index,:,:]
wa_cut = wa[0:end_index,:,:]
uvw=(ua**2+va**2+wa**2)**0.5
uvw_cut=uvw[1:end_index,:,:]
uv=(ua**2+va**2)**0.5
uv_cut=uv[0:end_index,:,:]
th_cut=pottemp[0:end_index,:,:]
tk_cut=tk[0:end_index,:,:]
rh_cut=rh[0:end_index,:,:]

# %% HORIZONTAL WIND SPEED
z_cross = vertcross(uv_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)
th_cross = vertcross(th_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)

ua_np = np.ma.copy(to_np(z_cross))

ter_line = interpline(ter, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)

PBLH_line = interpline(PBLH, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)
PBLH_line_ter=PBLH_line+ter_line

lats, lons = latlon_coords(ua)
cart_proj = get_cartopy(ua)

fig = pyplot.figure(figsize=(12,4))
ax_cross = pyplot.axes()
dbz_levels = np.arange(0,15,0.1) #total wind speed luk 09/12/2020 northerly foehn
xs = np.arange(0, z_cross.shape[-1], 1)
ys = to_np(z_cross.coords["vertical"])
dbz_contours = ax_cross.contourf(xs,ys,to_np(z_cross),levels=dbz_levels,extend="max",cmap="rainbow")
CS =ax_cross.contour(xs,ys,z_cross,levels=[i for i in np.arange(0,30,2)])
ax_cross.clabel(CS, inline=True, fontsize=12)

interval_x=10
interval_y=5

#add colorbar
cb_dbz = fig.colorbar(dbz_contours, ax=ax_cross)
cb_dbz.ax.tick_params(labelsize=15)
cb_dbz.ax.set_ylabel('Horizontal wind speed (m/s)', fontsize=15)
# Fill in the mountain area
ht_fill = ax_cross.fill_between(xs, 0, to_np(ter_line),facecolor="saddlebrown")

# Set the x-ticks to use latitude and longitude labels
coord_pairs = to_np(z_cross.coords["xy_loc"])
x_ticks = np.arange(coord_pairs.shape[0])
x_labels = [pair.latlon_str() for pair in to_np(coord_pairs)]

# Set the desired number of x ticks below
num_ticks = 10
thin = int((len(x_ticks) / num_ticks) + .5)
ax_cross.set_xticks(x_ticks[::thin])
ax_cross.set_xticklabels(x_labels[::thin], rotation=45, fontsize=10)

# Set the x-axis and  y-axis labels
ax_cross.set_xlabel("Latitude, Longitude", fontsize=15)
ax_cross.set_ylabel("Height (m.a.s.l.)", fontsize=15)
ax_cross.tick_params(labelsize=15)
pyplot.ylim(0,5000)
# pyplot.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_luk_09_12\Figure\xz_cross_section\paper_xz_13h_uv_5000.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
pyplot.show()
# %% TEMPERATURE
z_cross = vertcross(tk_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)
th_cross = vertcross(tk_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)

ua_np = np.ma.copy(to_np(z_cross))

ter_line = interpline(ter, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)

PBLH_line = interpline(PBLH, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)
PBLH_line_ter=PBLH_line+ter_line

lats, lons = latlon_coords(ua)
cart_proj = get_cartopy(ua)

fig = pyplot.figure(figsize=(12,4))
ax_cross = pyplot.axes()
dbz_levels = np.arange(250,275,1) #luk 09/12/2020 northerly foehn
xs = np.arange(0, z_cross.shape[-1], 1)
ys = to_np(z_cross.coords["vertical"])
dbz_contours = ax_cross.contourf(xs,ys,to_np(z_cross),levels=dbz_levels,extend="max",cmap="rainbow")
CS =ax_cross.contour(xs,ys,z_cross,levels=[i for i in np.arange(240,295,2)])
ax_cross.clabel(CS, inline=True, fontsize=12)

interval_x=10
interval_y=5

#add colorbar
cb_dbz = fig.colorbar(dbz_contours, ax=ax_cross)
cb_dbz.ax.tick_params(labelsize=15)
cb_dbz.ax.set_ylabel('Temperature (K)', fontsize=15)
# Fill in the mountain area
ht_fill = ax_cross.fill_between(xs, 0, to_np(ter_line),facecolor="saddlebrown")

# Set the x-ticks to use latitude and longitude labels
coord_pairs = to_np(z_cross.coords["xy_loc"])
x_ticks = np.arange(coord_pairs.shape[0])
x_labels = [pair.latlon_str() for pair in to_np(coord_pairs)]

# Set the desired number of x ticks below
num_ticks = 10
thin = int((len(x_ticks) / num_ticks) + .5)
ax_cross.set_xticks(x_ticks[::thin])
ax_cross.set_xticklabels(x_labels[::thin], rotation=45, fontsize=10)

# Set the x-axis and  y-axis labels
ax_cross.set_xlabel("Latitude, Longitude", fontsize=15)
ax_cross.set_ylabel("Height (m.a.s.l.)", fontsize=15)
ax_cross.tick_params(labelsize=15)
pyplot.ylim(0,5000)
# pyplot.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_luk_09_12\Figure\xz_cross_section\paper_xz_13h_tk_5000.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
pyplot.show()
# %% RELATIVE HUMIDITY
z_cross = vertcross(rh_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)
th_cross = vertcross(rh_cut, ht_cut, wrfin=wrf_file,start_point=cross_start,end_point=cross_end,latlon=True, meta=True)

ua_np = np.ma.copy(to_np(z_cross))

ter_line = interpline(ter, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)

PBLH_line = interpline(PBLH, wrfin=wrf_file, start_point=cross_start,
                      end_point=cross_end)
PBLH_line_ter=PBLH_line+ter_line

lats, lons = latlon_coords(ua)
cart_proj = get_cartopy(ua)

fig = pyplot.figure(figsize=(12,4))
ax_cross = pyplot.axes()
dbz_levels = np.arange(30,100,1)
xs = np.arange(0, z_cross.shape[-1], 1)
ys = to_np(z_cross.coords["vertical"])
dbz_contours = ax_cross.contourf(xs,ys,to_np(z_cross),levels=dbz_levels,extend="max",cmap="rainbow")
CS=ax_cross.contour(xs,ys,z_cross,levels=[i for i in np.arange(20,100,10)])
ax_cross.clabel(CS, inline=True, fontsize=12)

interval_x=10
interval_y=5

#add colorbar
cb_dbz = fig.colorbar(dbz_contours, ax=ax_cross)
cb_dbz.ax.tick_params(labelsize=15)
cb_dbz.ax.set_ylabel('Relative humidity (%)', fontsize=15)

# Fill in the mountain area
ht_fill = ax_cross.fill_between(xs, 0, to_np(ter_line),facecolor="saddlebrown")

# Set the x-ticks to use latitude and longitude labels
coord_pairs = to_np(z_cross.coords["xy_loc"])
x_ticks = np.arange(coord_pairs.shape[0])
x_labels = [pair.latlon_str() for pair in to_np(coord_pairs)]

# Set the desired number of x ticks below
num_ticks = 10
thin = int((len(x_ticks) / num_ticks) + .5)
ax_cross.set_xticks(x_ticks[::thin])
ax_cross.set_xticklabels(x_labels[::thin], rotation=45, fontsize=10)

# Set the x-axis and  y-axis labels
ax_cross.set_xlabel("Latitude, Longitude", fontsize=15)
ax_cross.set_ylabel("Height (m.a.s.l.)", fontsize=15)
ax_cross.tick_params(labelsize=15)
pyplot.ylim(0,5000)
# pyplot.savefig(r"H:\kristian\WRF_RESULT\WRF_COSMO1\WRF_spatial\WRF_SH_luk_09_12\Figure\xz_cross_section\xz_09h_rh_5000_mid.jpeg", format="jpeg", bbox_inches="tight",dpi=300)
pyplot.show()